import 'dart:async';

import 'package:angel_framework/angel_framework.dart'
    show Angel, RequestHandler, RequestContext, ResponseContext;
import 'package:daggerito/daggerito.dart';
import 'package:routex_web/routex_web.dart';

typedef HttpServerRequestFactory = HttpServerRequest Function(
    RequestContext, ResponseContext);

class AngelRoutexConfigurer {
  final Router router;
  final Angel app;
  final String method;
  final String path;
  final HttpServerRequestFactory requestFactory;
  final Component appComponent;

  RequestHandler get requestHandler => (req, res) async {
        try {
          final result =
              await router.handle(requestFactory(req, res), appComponent).run();
          if (result != -1) {
            res.write(result);
          } else {
            print("");
          }
        } catch (error) {
          if (error is ResponseStatusException) {
            res.statusCode = error.statusCode;
            res.write(error.toString());
          } else {
            res.write(error.toString());
          }
        }
      };

  AngelRoutexConfigurer({
    this.router,
    this.app,
    this.method = '*',
    this.path = '/*',
    this.requestFactory,
    this.appComponent,
  }) {
    RoutexAction.registerFactory(
      (handler) {
        if (handler is RoutingContextRequestHandler) {
//          print('3');
          return _RoutingContextRequestHandlerWrapper(handler);
        } else {
          return null;
        }
      },
    );
    RoutexAction.registerFactory(
      (handler) {
        if (handler is RequestHandler) {
//          print('5');
          return _RequestHandlerWrapper(handler);
        } else {
          return null;
        }
      },
    );
  }

  void configure() {
    app.addRoute(method, path, requestHandler);
  }
}

class _RoutingContextRequestHandlerWrapper implements Handler<RoutingContext> {
  final RoutingContextRequestHandler _handler;

  _RoutingContextRequestHandlerWrapper(this._handler);

  @override
  FutureOr handle(RoutingContext context) async {
    final requestHandler = await _handler(context);
    await requestHandler(context.container.resolve<RequestContext>(null,DependencyScope.app), context.container.resolve<ResponseContext>(null,DependencyScope.app));
    context.response().end(-1);
  }
}

typedef RoutingContextRequestHandler = FutureOr<RequestHandler> Function(
    RoutingContext);

class _RequestHandlerWrapper implements Handler<RoutingContext> {
  final RequestHandler _handler;

  _RequestHandlerWrapper(this._handler);

  @override
  FutureOr handle(RoutingContext context) async {
    await _handler(context.container.resolve<RequestContext>(null,DependencyScope.app), context.container.resolve<ResponseContext>(null,DependencyScope.app));
    context.response().end(-1);
  }
}
